#!/bin/env python3

import ctypes
from ctypes import *
from struct import pack
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
from enum import Enum

from application_classify import *

class SoftwareInstallControlMode(Enum):
    CLOSE_MODE = 0
    WHITELIST_MODE = 1
    BLACKLIST_MODE = 2

class SoftwareInstallControlType(Enum):
    WHITELIST = 1
    BLACKLIST = 2

class SoftwareUninstallControlMode(Enum):
    CLOSE_MODE = 0
    OPEN_MODE = 1

class SoftwareInstallControl:
    """
    <node>
        <interface name='com.kylin.kysdk.applicationsec.antiinstall'>
            <method name='SetMode' >
                <arg type='i' direction='in' name='mode'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetMode' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddBlackPackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddWhitePackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelBlackPackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <method name='DelWhitePackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddBlackPackagesList' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddWhitePackagesList' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='PackageIsInBlackList' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='PackageIsInWhilteList' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetBlackPackageList' >
                <arg type='as' direction='out' name='packagelist'/>
            </method>
            <method name='GetWhitePackageList' >
                <arg type='as' direction='out' name='packagelist'/>
            </method>
            <method name='ClearPackageList' >
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.bwlist_object = CDLL("libkylin_bwlist.so")
        self._black = create_string_buffer(b"black")
        self._white = create_string_buffer(b"white")

    def SetMode(self, mode):
        if self.bwlist_object.kylin_bwlist_create() == -1:
            return -1
        else:
            if mode == SoftwareInstallControlMode.CLOSE_MODE.value:
                return self.bwlist_object.kylin_bwlist_set_root_distinguish(0)
            elif mode == SoftwareInstallControlMode.WHITELIST_MODE.value:
                return self.bwlist_object.kylin_bwlist_set_root_distinguish(1)
            elif mode == SoftwareInstallControlMode.BLACKLIST_MODE.value:
                return self.bwlist_object.kylin_bwlist_set_root_distinguish(2)
            else:
                return -1

    def GetMode(self):
        rc = self.bwlist_object.kylin_bwlist_get_root_distinguish()
        if rc == -1:
            return SoftwareInstallControlMode.CLOSE_MODE.value
        return rc

    def AddBlackPackage(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_add(self._black, package)
    
    def AddWhitePackage(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_add(self._white, package)

    def DelBlackPackage(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_del(self._black, package)

    def DelWhitePackage(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_del(self._white, package)

    def AddBlackPackagesList(self, package_list):
        for package in package_list:
            package = create_string_buffer(package.encode())
            ret = self.bwlist_object.kylin_bwlist_add(self._black, package)
            if ret == -1:
                return -1
        return 0
    
    def AddWhitePackagesList(self, package_list):
        for package in package_list:
            package = create_string_buffer(package.encode())
            ret = self.bwlist_object.kylin_bwlist_add(self._white, package)
            if ret == -1:
                return -1
        return 0

    def PackageIsInBlackList(self, package):
        count = ctypes.c_int(0)
        self.bwlist_object.kylin_bwlist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        packagelist =  self.bwlist_object.kylin_bwlist_read_all_node_with_uid(self._black, byref(count))
        for p in range(0, count.value):
            if(str(packagelist[p], 'UTF-8') == package):
                return 1
        
        return 0

    def PackageIsInWhilteList(self, package):
        count = ctypes.c_int(0)
        self.bwlist_object.kylin_bwlist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        packagelist =  self.bwlist_object.kylin_bwlist_read_all_node_with_uid(self._white, byref(count))
        for p in range(0, count.value):
            if(str(packagelist[p], 'UTF-8') == package):
                return 1
        
        return 0

    def GetBlackPackageList(self):
        packagelist=[]
        count = ctypes.c_int(0)
        self.bwlist_object.kylin_bwlist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        plist =  self.bwlist_object.kylin_bwlist_read_all_node_with_uid(self._black, byref(count))
        for p in range(0, count.value):
            packagelist.append(str(plist[p], "UTF-8"))
        return packagelist

    def GetWhitePackageList(self):
        packagelist=[]
        count = ctypes.c_int(0)
        self.bwlist_object.kylin_bwlist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        plist =  self.bwlist_object.kylin_bwlist_read_all_node_with_uid(self._white, byref(count))
        for p in range(0, count.value):
            packagelist.append(str(plist[p], "UTF-8"))
        return packagelist

    def ClearPackageList(self):
        return self.bwlist_object.kylin_bwlist_clear()

class SoftwareUninstallControl:
    """
    <node>
        <interface name='com.kylin.kysdk.applicationsec.antiuninstall'>
            <method name='SetMode' >
                <arg type='i' direction='in' name='mode'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetMode' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddBlackPackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddWhitePackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelBlackPackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelWhitePackage' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddBlackPackagesList' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='AddWhitePackagesList' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='PackageIsInBlackList' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='PackageIsInWhilteList' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetBlackPackageList' >
                <arg type='as' direction='out' name='packagelist'/>
            </method>
            <method name='GetWhitePackageList' >
                <arg type='as' direction='out' name='packagelist'/>
            </method>
            <method name='ClearPackageList' >
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.udeblist_object = CDLL("libkylin_bwlist.so")
        self._black = create_string_buffer(b"black")
        self._white = create_string_buffer(b"white")

    def SetMode(self, mode):
        if self.udeblist_object.kylin_udeblist_create() == -1:
            return -1
        else:
            if mode == SoftwareInstallControlMode.CLOSE_MODE.value:
                return self.udeblist_object.kylin_udeblist_set_root_distinguish(0)
            elif mode == SoftwareInstallControlMode.WHITELIST_MODE.value:
                return self.udeblist_object.kylin_udeblist_set_root_distinguish(1)
            elif mode == SoftwareInstallControlMode.BLACKLIST_MODE.value:
                return self.udeblist_object.kylin_udeblist_set_root_distinguish(2)
            else:
                return -1

    def GetMode(self):
        rc = self.udeblist_object.kylin_udeblist_get_root_distinguish()
        if rc == -1:
            return SoftwareInstallControlMode.CLOSE_MODE.value
        return rc

    def AddBlackPackage(self, package):
        package = create_string_buffer(package.encode())
        return self.udeblist_object.kylin_udeblist_add(self._black, package)

    def AddWhitePackage(self, package):
        package = create_string_buffer(package.encode())
        return self.udeblist_object.kylin_udeblist_add(self._white, package)

    def DelBlackPackage(self, package):
        package = create_string_buffer(package.encode())
        return self.udeblist_object.kylin_udeblist_del(self._black, package)

    def DelWhitePackage(self, package):
        package = create_string_buffer(package.encode())
        return self.udeblist_object.kylin_udeblist_del(self._white, package)

    def AddBlackPackagesList(self, package_list):
        for package in package_list:
            package = create_string_buffer(package.encode())
            ret = self.udeblist_object.kylin_udeblist_add(self._black, package)
            if ret == -1:
                return -1
        return 0

    def AddWhitePackagesList(self, package_list):
        for package in package_list:
            package = create_string_buffer(package.encode())
            ret = self.udeblist_object.kylin_udeblist_add(self._white, package)
            if ret == -1:
                return -1
        return 0

    def PackageIsInBlackList(self, package):
        count = ctypes.c_int(0)
        self.udeblist_object.kylin_udeblist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        packagelist =  self.udeblist_object.kylin_udeblist_read_all_node_with_uid(self._black, byref(count))
        for p in range(0, count.value):
            if(str(packagelist[p], 'UTF-8') == package):
                return 1
        
        return 0

    def PackageIsInWhilteList(self, package):
        count = ctypes.c_int(0)
        self.udeblist_object.kylin_udeblist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        packagelist =  self.udeblist_object.kylin_udeblist_read_all_node_with_uid(self._white, byref(count))
        for p in range(0, count.value):
            if(str(packagelist[p], 'UTF-8') == package):
                return 1
        
        return 0

    def GetBlackPackageList(self):
        packagelist=[]
        count = ctypes.c_int(0)
        self.udeblist_object.kylin_udeblist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        plist =  self.udeblist_object.kylin_udeblist_read_all_node_with_uid(self._black, byref(count))
        for p in range(0, count.value):
            packagelist.append(str(plist[p], "UTF-8"))
        return packagelist

    def GetWhitePackageList(self,):
        packagelist=[]
        count = ctypes.c_int(0)
        self.udeblist_object.kylin_udeblist_read_all_node_with_uid.restype = POINTER(ctypes.c_char_p)

        plist =  self.udeblist_object.kylin_udeblist_read_all_node_with_uid(self._white, byref(count))
        for p in range(0, count.value):
            packagelist.append(str(plist[p], "UTF-8"))
        return packagelist

    def ClearPackageList(self):
        return self.udeblist_object.kylin_udeblist_clear()


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.kysdk.applicationsec", SoftwareInstallControl(), SoftwareUninstallControl(), classify())
    loop.run()